"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Bifurcation_1 = __importDefault(require("./Bifurcation"));
const Node_1 = __importDefault(require("./Node"));
const Connection_1 = require("../../api/contract/Connection");
class Graph extends Node_1.default {
    constructor() {
        super(...arguments);
        this.graph = {};
    }
    addNodeToGraph(node) {
        if (this.checkIfNodeExistInGraph(node.id)) {
            throw new Error(`Error inserting node ${node.id}. ID duplicated`);
        }
        if (this.checkIfNodeExistInAnyChildGraph(node.id)) {
            throw new Error(`Error inserting node ${node.id}. ID duplicated`);
        }
        this.graph = Object.assign(Object.assign({}, this.graph), { [node.id]: node });
    }
    checkIfNodeExistInGraph(nodeId) {
        return this.graph.hasOwnProperty(nodeId);
    }
    checkIfNodeExistInAnyChildGraph(nodeId) {
        let result = null;
        const keys = Object.keys(this.graph);
        for (const key of keys) {
            const value = this.graph[key];
            result = value.navigate(nodeId);
            if (result !== null && result.id === nodeId) {
                return true;
            }
        }
        return false;
    }
    addNodeToGraphIfNotExist(node) {
        if (!this.checkIfNodeExistInGraph(node.id)) {
            this.addNodeToGraph(node);
        }
    }
    addConnectionInNodeGraph(nodeId, connection) {
        if (this.checkIfNodeExistInGraph(nodeId)) {
            this.graph[nodeId].addOrRemplaceConnection(connection);
        }
    }
    nexFrom(nodeFrom, nodeTo, forceSync = false) {
        const nodeFromConnections = new Connection_1.Connection();
        nodeFromConnections.id = "continue";
        nodeFromConnections.stepTo = nodeTo.id;
        nodeFromConnections.forceSync = forceSync;
        if (!this.checkIfNodeExistInGraph(nodeTo.id)) {
            this.addNodeToGraph(nodeTo);
        }
        if (this.checkIfNodeExistInGraph(nodeFrom.id)) {
            this.addConnectionInNodeGraph(nodeFrom.id, nodeFromConnections);
        }
        else {
            nodeFrom.addOrRemplaceConnection(nodeFromConnections);
            this.addNodeToGraph(nodeFrom);
        }
    }
    linkFrom(
    // TODO fix pending: doesnt work without sync
    nodeFrom, nodeTo, linkId, forceSync = true) {
        const nodeFromConnections = new Connection_1.Connection();
        nodeFromConnections.id = linkId;
        nodeFromConnections.stepTo = nodeTo.id;
        nodeFromConnections.forceSync = forceSync;
        nodeFrom.addOrRemplaceConnection(nodeFromConnections);
        //this.addNodeToGraph(nodeTo);
        if (!this.checkIfNodeExistInGraph(nodeTo.id)) {
            this.addNodeToGraph(nodeTo);
        }
        if (this.checkIfNodeExistInGraph(nodeFrom.id)) {
            this.addConnectionInNodeGraph(nodeFrom.id, nodeFromConnections);
        }
        else {
            nodeFrom.addOrRemplaceConnection(nodeFromConnections);
            this.addNodeToGraph(nodeFrom);
        }
    }
    bifurcationFrom(nodeFrom, nodeOnTrue, nodeOnFalse, condition, forceSync = false) {
        const bifurcationNode = new Bifurcation_1.default();
        bifurcationNode.condition = condition;
        bifurcationNode.id = `${nodeFrom.id}-${nodeOnTrue.id}-${nodeOnFalse.id}`;
        const nodeFromConnections = new Connection_1.Connection();
        nodeFromConnections.id = "continue";
        nodeFromConnections.stepTo = bifurcationNode.id;
        nodeFromConnections.forceSync = forceSync;
        if (this.checkIfNodeExistInGraph(nodeFrom.id)) {
            this.addConnectionInNodeGraph(nodeFrom.id, nodeFromConnections);
        }
        else {
            nodeFrom.addOrRemplaceConnection(nodeFromConnections);
            this.addNodeToGraph(nodeFrom);
        }
        const onTrueConnection = new Connection_1.Connection();
        onTrueConnection.id = "on_true_connection";
        onTrueConnection.forceSync = forceSync;
        onTrueConnection.stepTo = nodeOnTrue.id;
        const onFalseConnection = new Connection_1.Connection();
        onFalseConnection.id = "on_false_connection";
        onFalseConnection.forceSync = forceSync;
        onFalseConnection.stepTo = nodeOnFalse.id;
        bifurcationNode.addOrRemplaceConnection(onTrueConnection);
        bifurcationNode.addOrRemplaceConnection(onFalseConnection);
        this.addNodeToGraph(bifurcationNode);
        this.addNodeToGraphIfNotExist(nodeOnTrue);
        this.addNodeToGraphIfNotExist(nodeOnFalse);
    }
    getNexStepConnection() {
        const continueConnection = this.connections["continue"];
        return continueConnection;
    }
    navigate(stepId) {
        let result = null;
        const keys = Object.keys(this.graph);
        for (const key of keys) {
            const value = this.graph[key];
            result = value.navigate(stepId);
            if (result !== null && result.id === stepId) {
                break;
            }
        }
        return result;
    }
}
exports.default = Graph;
